@@ -25,11 +25,13 @@ from __future__ import absolute_import, division
 
 import inspect
 import os
+import platform
 import socket
-import string
 import struct
 import sys
+import tokenize
 from types import MethodType as _MethodType
+import warnings
 
 from io import TextIOBase, IOBase
 
@@ -39,6 +41,41 @@ if sys.version_info < (3, 0):
 else:
     _PY3 = True
 
+if sys.version_info >= (3, 5, 0):
+    _PY35PLUS = True
+else:
+    _PY35PLUS = False
+
+if sys.version_info >= (3, 7, 0):
+    _PY37PLUS = True
+else:
+    _PY37PLUS = False
+
+if platform.python_implementation() == 'PyPy':
+    _PYPY = True
+else:
+    _PYPY = False
+
+
+
+def _shouldEnableNewStyle():
+    """
+    Returns whether or not we should enable the new-style conversion of
+    old-style classes. It inspects the environment for C{TWISTED_NEWSTYLE},
+    accepting an empty string, C{no}, C{false}, C{False}, and C{0} as falsey
+    values and everything else as a truthy value.
+
+    @rtype: L{bool}
+    """
+    value = os.environ.get('TWISTED_NEWSTYLE', '')
+
+    if value in ['', 'no', 'false', 'False', '0']:
+        return False
+    else:
+        return True
+
+
+_EXPECT_NEWSTYLE = _PY3 or _shouldEnableNewStyle()
 
 
 def currentframe(n=0):
@@ -61,12 +98,27 @@ def currentframe(n=0):
 
 
 def inet_pton(af, addr):
+    """
+    Emulator of L{socket.inet_pton}.
+
+    @param af: An address family to parse; C{socket.AF_INET} or
+        C{socket.AF_INET6}.
+    @type af: L{int}
+
+    @param addr: An address.
+    @type addr: native L{str}
+
+    @return: The binary packed version of the passed address.
+    @rtype: L{bytes}
+    """
+    if not addr:
+        raise ValueError("illegal IP address string passed to inet_pton")
     if af == socket.AF_INET:
         return socket.inet_aton(addr)
     elif af == getattr(socket, 'AF_INET6', 'AF_INET6'):
-        if [x for x in addr if x not in string.hexdigits + ':.']:
-            raise ValueError("Illegal characters: %r" % (''.join(x),))
-
+        if '%' in addr and (addr.count('%') > 1 or addr.index("%") == 0):
+            raise ValueError("illegal IP address string passed to inet_pton")
+        addr = addr.split('%')[0]
         parts = addr.split(':')
         elided = parts.count('')
         ipv4Component = '.' in parts[-1]
@@ -106,6 +158,8 @@ def inet_pton(af, addr):
     else:
         raise socket.error(97, 'Address family not supported by protocol')
 
+
+
 def inet_ntop(af, addr):
     if af == socket.AF_INET:
         return socket.inet_ntoa(addr)
@@ -187,11 +241,8 @@ def execfile(filename, globals, locals=None):
     """
     if locals is None:
         locals = globals
-    fin = open(filename, "rbU")
-    try:
+    with open(filename, "rb") as fin:
         source = fin.read()
-    finally:
-        fin.close()
     code = compile(source, filename, "exec")
     exec(code, globals, locals)
 
@@ -228,6 +279,7 @@ def comparable(klass):
     if not _PY3:
         return klass
 
+
     def __eq__(self, other):
         c = self.__cmp__(other)
         if c is NotImplemented:
@@ -422,7 +474,7 @@ Note that on Python 3, re-raised exceptions will be mutated, with their
 C{__traceback__} attribute being set.
 
 @param exception: The exception instance.
-@param traceback: The traceback to use, or C{None} indicating a new traceback.
+@param traceback: The traceback to use, or L{None} indicating a new traceback.
 """
 
 
@@ -446,10 +498,6 @@ if _PY3:
         return ("%d" % i).encode("ascii")
 
 
-    # Ideally we would use memoryview, but it has a number of differences from
-    # the Python 2 buffer() that make that impractical
-    # (http://bugs.python.org/issue15945, incompatibility with pyOpenSSL due to
-    # PyArg_ParseTuple differences.)
     def lazyByteSlice(object, offset=0, size=None):
         """
         Return a copy of the given bytes-like object.
@@ -464,10 +512,11 @@ if _PY3:
         @param size: Optional, if an C{int} is given limit the length of copy
             to this size.
         """
+        view = memoryview(object)
         if size is None:
-            return object[offset:]
+            return view[offset:]
         else:
-            return object[offset:(offset + size)]
+            return view[offset:(offset + size)]
 
 
     def networkString(s):
@@ -482,7 +531,6 @@ else:
     def intToBytes(i):
         return b"%d" % i
 
-
     lazyByteSlice = buffer
 
     def networkString(s):
@@ -568,24 +616,30 @@ if _PY3:
     def iteritems(d):
         return d.items()
 
+
     def itervalues(d):
         return d.values()
 
+
     def items(d):
         return list(d.items())
 
+    range = range
     xrange = range
     izip = zip
 else:
     def iteritems(d):
         return d.iteritems()
 
+
     def itervalues(d):
         return d.itervalues()
 
+
     def items(d):
         return d.items()
 
+    range = xrange
     xrange = xrange
     from itertools import izip
     izip # shh pyflakes
@@ -612,12 +666,27 @@ Return a list of the items of C{d}.
 @rtype: L{list}
 """
 
+def _keys(d):
+    """
+    Return a list of the keys of C{d}.
+
+    @type d: L{dict}
+    @rtype: L{list}
+    """
+    if _PY3:
+        return list(d.keys())
+    else:
+        return d.keys()
+
 
 
 def bytesEnviron():
     """
     Return a L{dict} of L{os.environ} where all text-strings are encoded into
     L{bytes}.
+
+    This function is POSIX only; environment variables are always text strings
+    on Windows.
     """
     if not _PY3:
         # On py2, nothing to do.
@@ -654,16 +723,132 @@ def _constructMethod(cls, name, self):
 
 
 
-from twisted.python.versions import Version
-from twisted.python.deprecate import deprecatedModuleAttribute
+if _PY3:
+    from base64 import encodebytes as _b64encodebytes
+    from base64 import decodebytes as _b64decodebytes
+else:
+    from base64 import encodestring as _b64encodebytes
+    from base64 import decodestring as _b64decodebytes
 
-from collections import OrderedDict
 
-deprecatedModuleAttribute(
-    Version("Twisted", 15, 5, 0),
-    "Use collections.OrderedDict instead.",
-    "twisted.python.compat",
-    "OrderedDict")
+
+def _bytesChr(i):
+    """
+    Like L{chr} but always works on ASCII, returning L{bytes}.
+
+    @param i: The ASCII code point to return.
+    @type i: L{int}
+
+    @rtype: L{bytes}
+    """
+    if _PY3:
+        return bytes([i])
+    else:
+        return chr(i)
+
+
+
+try:
+    from sys import intern
+except ImportError:
+    intern = intern
+
+
+
+def _coercedUnicode(s):
+    """
+    Coerce ASCII-only byte strings into unicode for Python 2.
+
+    In Python 2 C{unicode(b'bytes')} returns a unicode string C{'bytes'}. In
+    Python 3, the equivalent C{str(b'bytes')} will return C{"b'bytes'"}
+    instead. This function mimics the behavior for Python 2. It will decode the
+    byte string as ASCII. In Python 3 it simply raises a L{TypeError} when
+    passing a byte string. Unicode strings are returned as-is.
+
+    @param s: The string to coerce.
+    @type s: L{bytes} or L{unicode}
+
+    @raise UnicodeError: The input L{bytes} is not ASCII decodable.
+    @raise TypeError: The input is L{bytes} on Python 3.
+    """
+    if isinstance(s, bytes):
+        if _PY3:
+            raise TypeError("Expected str not %r (bytes)" % (s,))
+        else:
+            return s.decode('ascii')
+    else:
+        return s
+
+
+
+if _PY3:
+    unichr = chr
+    raw_input = input
+else:
+    unichr = unichr
+    raw_input = raw_input
+
+
+
+def _bytesRepr(bytestring):
+    """
+    Provide a repr for a byte string that begins with 'b' on both
+    Python 2 and 3.
+
+    @param bytestring: The string to repr.
+    @type bytestring: L{bytes}
+
+    @raise TypeError: The input is not L{bytes}.
+
+    @return: The repr with a leading 'b'.
+    @rtype: L{bytes}
+    """
+    if not isinstance(bytestring, bytes):
+        raise TypeError("Expected bytes not %r" % (bytestring,))
+
+    if _PY3:
+        return repr(bytestring)
+    else:
+        return 'b' + repr(bytestring)
+
+
+if _PY3:
+    _tokenize = tokenize.tokenize
+else:
+    _tokenize = tokenize.generate_tokens
+
+try:
+    from collections.abc import Sequence
+except ImportError:
+    from collections import Sequence
+
+
+
+def _get_async_param(isAsync=None, **kwargs):
+    """
+    Provide a backwards-compatible way to get async param value that does not
+    cause a syntax error under Python 3.7.
+
+    @param isAsync: isAsync param value (should default to None)
+    @type isAsync: L{bool}
+
+    @param kwargs: keyword arguments of the caller (only async is allowed)
+    @type kwargs: L{dict}
+
+    @raise TypeError: Both isAsync and async specified.
+
+    @return: Final isAsync param value
+    @rtype: L{bool}
+    """
+    if 'async' in kwargs:
+        warnings.warn(
+            "'async' keyword argument is deprecated, please use isAsync",
+            DeprecationWarning, stacklevel=2)
+    if isAsync is None and 'async' in kwargs:
+        isAsync = kwargs.pop('async')
+    if kwargs:
+        raise TypeError
+    return bool(isAsync)
 
 
 
@@ -689,6 +874,7 @@ __all__ = [
     "items",
     "iteritems",
     "itervalues",
+    "range",
     "xrange",
     "urllib_parse",
     "bytesEnviron",
@@ -696,4 +882,16 @@ __all__ = [
     "urlquote",
     "urlunquote",
     "cookielib",
+    "_keys",
+    "_b64encodebytes",
+    "_b64decodebytes",
+    "_bytesChr",
+    "_coercedUnicode",
+    "_bytesRepr",
+    "intern",
+    "unichr",
+    "raw_input",
+    "_tokenize",
+    "_get_async_param",
+    "Sequence",
 ]
